package sun.net.www.protocol.jar;

import java.security.Permission;
import java.net.SocketPermission;
import java.io.FilePermission;
import java.io.FileNotFoundException;
import sun.net.util.URLUtil;
import java.io.IOException;
import java.net.URLConnection;
import java.net.URL;
import java.util.jar.JarFile;
import java.util.HashMap;

class JarFileFactory implements URLJarFile.URLJarFileCloseController
{
    private static final HashMap<String, JarFile> fileCache;
    private static final HashMap<JarFile, URL> urlCache;
    private static final JarFileFactory instance;
    
    private JarFileFactory() {
    }
    
    public static JarFileFactory getInstance() {
        return JarFileFactory.instance;
    }
    
    URLConnection getConnection(JarFile key) throws IOException {
        URL url;
        synchronized (JarFileFactory.instance) {
            url = (URL)JarFileFactory.urlCache.get(key);
        }
        if (url != null) {
            return url.openConnection();
        }
        return null;
    }
    
    public JarFile get(URL url) throws IOException {
        return this.get(url, true);
    }
    
    JarFile get(URL url, boolean b) throws IOException {
        if (url.getProtocol().equalsIgnoreCase("file")) {
            JarFile str = url.getHost();
            if (str != null && !str.equals("") && !str.equalsIgnoreCase("localhost")) {
                url = new URL("file", "", "//" + str + url.getPath());
            }
        }
        JarFile str;
        if (b) {
            synchronized (JarFileFactory.instance) {
                str = this.getCachedJarFile(url);
            }
            if (str == null) {
                JarFile jarFile = URLJarFile.getJarFile(url, this);
                synchronized (JarFileFactory.instance) {
                    str = this.getCachedJarFile(url);
                    if (str == null) {
                        JarFileFactory.fileCache.put(URLUtil.urlNoFragString(url), jarFile);
                        JarFileFactory.urlCache.put(jarFile, url);
                        str = jarFile;
                    }
                    else if (jarFile != null) {
                        jarFile.close();
                    }
                }
            }
        }
        else {
            str = URLJarFile.getJarFile(url, this);
        }
        if (str == null) {
            throw new FileNotFoundException(url.toString());
        }
        return str;
    }
    
    @Override
    public void close(JarFile key) {
        synchronized (JarFileFactory.instance) {
            URL url = (URL)JarFileFactory.urlCache.remove(key);
            if (url != null) {
                JarFileFactory.fileCache.remove(URLUtil.urlNoFragString(url));
            }
        }
    }
    
    private JarFile getCachedJarFile(URL url) {
        assert !(!Thread.holdsLock(JarFileFactory.instance));
        JarFile jarFile = (JarFile)JarFileFactory.fileCache.get(URLUtil.urlNoFragString(url));
        if (jarFile != null) {
            Permission permission = this.getPermission(jarFile);
            if (permission != null) {
                SecurityManager securityManager = System.getSecurityManager();
                if (securityManager != null) {
                    try {
                        securityManager.checkPermission(permission);
                    }
                    catch (SecurityException ex) {
                        if (permission instanceof FilePermission && permission.getActions().indexOf("read") != -1) {
                            securityManager.checkRead(permission.getName());
                        }
                        else if (permission instanceof SocketPermission && permission.getActions().indexOf("connect") != -1) {
                            securityManager.checkConnect(url.getHost(), url.getPort());
                        }
                        else {
                            throw ex;
                        }
                    }
                }
            }
        }
        return jarFile;
    }
    
    private Permission getPermission(JarFile jarFile) {
        try {
            URLConnection connection = this.getConnection(jarFile);
            if (connection != null) {
                return connection.getPermission();
            }
        }
        catch (IOException ex) {}
        return null;
    }
    
    static {
        fileCache = new HashMap();
        urlCache = new HashMap();
        instance = new JarFileFactory();
    }
}
